@@ -181,6 +181,9 @@ private Q_SLOTS:
         QTest::newRow("vgn-trip-indoor-path")
             << s(SOURCE_DIR "/data/efa/trip-response-full-indoor-path-vgn.xml")
             << s(SOURCE_DIR "/data/efa/trip-response-full-indoor-path-vgn.json");
+        QTest::newRow("vrt-trip-alternative-notes")
+            << s(SOURCE_DIR "/data/efa/trip-response-full-vrt.xml")
+            << s(SOURCE_DIR "/data/efa/trip-response-full-vrt.json");
     }
 
     void testParseTrips()
